/***************************************************************************
 *   Copyright (C) 2004 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "camera.h"
 
CoglCamera3D::CoglCamera3D() {
	// constructor
	
	vRotationVector.setX( 5.0 );
	vRotationVector.setY( 0.0 );
	vRotationVector.setZ( 0.0 );

	double Phi = vRotationVector.getY() * M_PI / 180;
	double The = vRotationVector.getX() * M_PI / 180;

	vKameraViewPoint.setX( 10 * cos (Phi) * cos (The) );
	vKameraViewPoint.setY( 10 * sin (The) );
	vKameraViewPoint.setZ( 10 * sin (Phi) * cos (The) );
	oben = (The <= 0.5 * M_PI || The > 1.5 * M_PI) * 2 - 1;
}

CoglCamera3D::CoglCamera3D( float pos_x, float pos_y, float pos_z ) {
	// constructor

	vRotationVector.setX( 0.0 );
	vRotationVector.setY( 180.0 );
	vRotationVector.setZ( 0.0 );

	vKameraViewPoint.setX( pos_x );
	vKameraViewPoint.setY( pos_y );
	vKameraViewPoint.setZ( pos_z );

// 	double Phi = vRotationVector.getY() * M_PI / 180;
	double The = vRotationVector.getX() * M_PI / 180;
	oben = (The <= 0.5 * M_PI || The > 1.5 * M_PI) * 2 - 1;

}

CoglCamera3D::~CoglCamera3D() {
	// destructor
}

void CoglCamera3D::left() {
	// rotate the camera to the left
}

void CoglCamera3D::right() {
	// rotate the camera to the right
}

void CoglCamera3D::up() {
	// rotate the camera to the up
}

void CoglCamera3D::down() {
	// rotate the camera to the down
}

void CoglCamera3D::forward() {
	// move the camera to the forward
}	

void CoglCamera3D::backward() {
	// move the camera to the forward
}

void CoglCamera3D::setCamPos() {
    // set new camera position

    glMatrixMode (GL_MODELVIEW);

    if (mode == 0) {

        glLoadIdentity();
        gluLookAt (	vKameraViewPoint.getX(),
                        vKameraViewPoint.getY(),
                        vKameraViewPoint.getZ(), 
                        0,1,0, 0, oben, 0);

    }
    else {

        GLfloat SzeneRotation = 360.0f - (GLfloat)vRotationVector.getY();

        glRotatef((GLfloat)vRotationVector.getX(), 1.0, 0.0, 0.0);
        glRotatef(SzeneRotation, 0.0, 1.0, 0.0);
        glTranslatef(	-(GLfloat)vKameraViewPoint.getX(),
                        -(GLfloat)vKameraViewPoint.getY(),
                        -(GLfloat)vKameraViewPoint.getZ());

    }
}


/*!
    \fn CoglCamera3D::calcPosFreeCam()
 */
// calculate camera transformation for the free camera
void CoglCamera3D::calcPosFreeCam(double dMoveX, double dMoveY, double dRadius, int state)
{

    double Phi, The;

    switch (state) {
        case (Qt::LeftButton):
                vRotationVector.setX( vRotationVector.getX() + dMoveY );
                vRotationVector.setY( vRotationVector.getY() - dMoveX );
            break;
        case (Qt::MidButton):
            break;
        case (Qt::RightButton):
                Phi = vRotationVector.getY() * M_PI / 180;
                The = vRotationVector.getX() * M_PI / 180;

                vKameraViewPoint.setX( vKameraViewPoint.getX() + dMoveY * sin (Phi) * cos (The) );
                vKameraViewPoint.setY( vKameraViewPoint.getY() + dMoveY * sin (The) * sin (Phi) );
                vKameraViewPoint.setZ( vKameraViewPoint.getZ() + dMoveY * cos (Phi) );
            break;
        default:
            break;
    }

    if (vRotationVector.getX() < 0.0) vRotationVector.setX( vRotationVector.getX() + 360 );
    if (vRotationVector.getX() > 360.0) vRotationVector.setX( vRotationVector.getX() - 360 );
    if (vRotationVector.getY() < 0.0) vRotationVector.setY( vRotationVector.getY() + 360 );
    if (vRotationVector.getY() > 360.0) vRotationVector.setY( vRotationVector.getY() - 360 );
}


/*!
    \fn CoglCamera3D::UpdateMouseData();
 */
// updates the camera if mouse is moved
void CoglCamera3D::updateMouseData(double dMoveX, double dMoveY, double dRadius, int state)
{

    // centered camera
    if (mode == 0) {

        calcPosCenterCam( dMoveX, dMoveY, dRadius, state);

    }
    // free camera
    else {

        calcPosFreeCam( dMoveX, dMoveY, dRadius, state);

    }

}


/*!
    \fn CoglCamera3D::setCameraMode(int mode)
 */
// toggle the camera modes
void CoglCamera3D::setCameraMode(int mode)
{
	this->mode = mode;
}


/*!
    \fn CoglCamera3D::calcPosCenterCam(double dMoveX, double dMoveY, double dRadius)
 */
// calculate camera transformations for the center camera
void CoglCamera3D::calcPosCenterCam(double dMoveX, double dMoveY, double dRadius, int state)
{

    switch (state) {
        case (Qt::LeftButton):
            vRotationVector.setX( vRotationVector.getX() + dMoveY );
            vRotationVector.setY( vRotationVector.getY() - dMoveX );
            break;
        case (Qt::MidButton):
            break;
        case (Qt::RightButton):
            break;
        }

    if (vRotationVector.getX() < 0.0) vRotationVector.setX( vRotationVector.getX() + 360 );
    if (vRotationVector.getX() > 360.0) vRotationVector.setX( vRotationVector.getX() - 360 );
    if (vRotationVector.getY() < 0.0) vRotationVector.setY( vRotationVector.getY() + 360 );
    if (vRotationVector.getY() > 360.0) vRotationVector.setY( vRotationVector.getY() - 360 );

    double The,Phi;

    Phi = vRotationVector.getY() * M_PI / 180;
    The = vRotationVector.getX() * M_PI / 180;

    vKameraViewPoint.setX( dRadius * cos (Phi) * cos (The) );
    vKameraViewPoint.setY( dRadius * sin (The) );
    vKameraViewPoint.setZ( dRadius * sin (Phi) * cos (The) );
    oben = (The <= 0.5 * M_PI || The > 1.5 * M_PI) * 2 - 1;

}

/*!
    \fn CoglCamera3D::calcPosCenterCam(double dMoveX, double dMoveY, double dRadius)
 */
// set the viewpoint manual
void CoglCamera3D::setViewPoint( float x, float y, float z )
{
    vKameraViewPoint.setX( x );
    vKameraViewPoint.setY( y );
    vKameraViewPoint.setZ( z );
}
