/***************************************************************************
 *   Copyright (C) 2006 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "citeratornd.h"

CIteratorND::CIteratorND()
{
    this->dimension = 0;
    this->iteratorND = NULL;
    this->grid = NULL;
}

CIteratorND::CIteratorND( int dimension, CDtGrid *dtgrid )
{
    if ( (dtgrid != NULL) && (dimension != dtgrid->dimension) )
        cerr << "Iterator: Error! Different dimension of DT Grid!" << endl;

    this->grid = dtgrid;

    this->value = 0;

    this->dimension = dimension;
    if ( dimension > 1 )
        this->iteratorND = new CIteratorND( dimension -1, dtgrid->projND );
    else
        this->iteratorND = NULL;
}

CIteratorND::~CIteratorND()
{
    if ( this->iteratorND != NULL ) delete this->iteratorND;
}


/*!
    \fn CIteratorND::increment( CLocatorND loc )
 */
bool CIteratorND::increment( CLocatorND *loc )
{
    if ( this->grid == NULL) cerr << "IteratorND: Error! No DT Grid found!" << endl;

    loc->iValue++;

    if ( this->grid->parent == NULL ) {
        if ( loc->iValue >= grid->value.size() ) return false;
        else this->value = this->grid->value.at( loc->iValue );
    }
//     else {
//         if ( loc->iValue > grid->valueIndexPair.size() ) return false;
//         else this->valueIndexPair = this->grid->valueIndexPair.at( loc->iValue );
//     }

    if ( loc->Xn == this->grid->nCoord.at( loc->iCoord+1 ) ) {

        // update locator
        loc->iCoord += 2;
        if ( loc->iCoord >= this->grid->nCoord.size() ) return false;
        else loc->Xn = this->grid->nCoord.at( loc->iCoord );

        if ( (this->grid->projND != NULL) && 
             ((loc->loc->iValue+1) < this->grid->projND->valueIndexPair.size()) &&
             (this->grid->projND->valueIndexPair[loc->loc->iValue+1].iCoord == loc->iCoord) )
            this->iteratorND->increment( loc->loc );
    }
    else {
        loc->Xn++;
    }

    return true;
}


/*!
    \fn CIteratorND::initializeLocator( CLocatorND *loc )
 */
void CIteratorND::initializeLocator( CLocatorND *loc )
{
    if ( this->grid == NULL) cerr << "IteratorND: Error! No DT Grid found!" << endl;

    loc->iValue = 0;
    loc->iCoord = 0;
    loc->Xn = this->grid->nCoord.at(0);

    if ( this->grid->parent == NULL ) {
        this->value = this->grid->value.at( loc->iValue );
    }

    if ( this->grid->projND != NULL ) {
        this->iteratorND->initializeLocator( loc->loc );
    }
}
