/***************************************************************************
 *   Copyright (C) 2005 Chemnitz University of Technology                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "cube.h"

CCube::CCube() {

    this->flag = CT_EMPTY;

}

CCube::CCube(CUBE_TYPE flag, double value) {

    this->flag = flag;
    this->value = value;
}

CCube::~CCube() {


}


//////////////////////////////////////////////////////////////////////
// Drawing
//////////////////////////////////////////////////////////////////////

double CCube::Draw(CVertex3D *p1, CVertex3D *p2, CColor4f *color) {

    struct timeval  tp1,tp2;            // process cpu time in seconds (tv_sec) and micro seconds (tv_usec)

    gettimeofday( &tp1, NULL );

    // Testen, ob Cube berhaupt zu zeichnen ist
    if (this->flag == CT_EMPTY) {
        gettimeofday( &tp2, NULL );
        return (tp2.tv_sec -tp1.tv_sec) * 1000000 + (tp2.tv_usec - tp1.tv_usec);
    }

    gettimeofday( &tp2, NULL );

    double value = this->value;

    // Flchen zeichnen
    // ----------------

        // Transparenz einschalten und Licht ausschalten
        //glEnable(GL_BLEND);
        //glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

        // Farbe festlegen
        glColor3f(color->getRed(), color->getGreen(), color->getBlue());
        //glColor4f(color->r, color->g, color->b, 0.5f);

        // Offset fr den Cube (bessere Darstellung der Kanten)
        glEnable(GL_POLYGON_OFFSET_FILL);
        glPolygonOffset(1.f, 1.f);

            glBegin(GL_POLYGON);
                glNormal3f(  -1.0,  0.0,  0.0);
                glVertex3f(  p1->getX(), p1->getY(), p1->getZ());
                glVertex3f(  p1->getX(), p1->getY(), p2->getZ());
                glVertex3f(  p1->getX(), p2->getY(), p2->getZ());
                glVertex3f(  p1->getX(), p2->getY(), p1->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d( 1.0,  0.0,  0.0);
                glVertex3d( p2->getX(), p2->getY(), p1->getZ());
                glVertex3d( p2->getX(), p2->getY(), p2->getZ());
                glVertex3d( p2->getX(), p1->getY(), p2->getZ());
                glVertex3d( p2->getX(), p1->getY(), p1->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0,  -1.0,  0.0);
                glVertex3d( p1->getX(), p1->getY(), p1->getZ());
                glVertex3d( p2->getX(), p1->getY(), p1->getZ());
                glVertex3d( p2->getX(), p1->getY(), p2->getZ());
                glVertex3d( p1->getX(), p1->getY(), p2->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0, 1.0,  0.0);
                glVertex3d( p2->getX(), p2->getY(), p1->getZ());
                glVertex3d( p1->getX(), p2->getY(), p1->getZ());
                glVertex3d( p1->getX(), p2->getY(), p2->getZ());
                glVertex3d( p2->getX(), p2->getY(), p2->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0,  0.0, -1.0);
                glVertex3d( p1->getX(), p1->getY(), p1->getZ());
                glVertex3d( p1->getX(), p2->getY(), p1->getZ());
                glVertex3d( p2->getX(), p2->getY(), p1->getZ());
                glVertex3d( p2->getX(), p1->getY(), p1->getZ());
            glEnd();

            glBegin(GL_POLYGON);
                glNormal3d(  0.0,  0.0, 1.0);
                glVertex3d( p2->getX(), p1->getY(), p2->getZ());
                glVertex3d( p2->getX(), p2->getY(), p2->getZ());
                glVertex3d( p1->getX(), p2->getY(), p2->getZ());
                glVertex3d( p1->getX(), p1->getY(), p2->getZ());
            glEnd();

        glDisable(GL_POLYGON_OFFSET_FILL);


// Kanten zeichnen
// ---------------

        glColor3d(color->getRed() / 1.8f, color->getGreen() / 1.8f, color->getBlue() / 1.8f);

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p1->getZ());
            glVertex3d(p1->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p1->getZ());
            glVertex3d(p2->getX(), p1->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p1->getZ());
            glVertex3d(p1->getX(), p2->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p1->getY(), p1->getZ());
            glVertex3d(p2->getX(), p2->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p1->getY(), p1->getZ());
            glVertex3d(p2->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p2->getY(), p1->getZ());
            glVertex3d(p2->getX(), p2->getY(), p1->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p2->getY(), p1->getZ());
            glVertex3d(p1->getX(), p2->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p2->getY(), p1->getZ());
            glVertex3d(p2->getX(), p2->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p2->getY(), p2->getZ());
            glVertex3d(p1->getX(), p2->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p2->getX(), p2->getY(), p2->getZ());
            glVertex3d(p2->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p2->getY(), p2->getZ());
            glVertex3d(p1->getX(), p1->getY(), p2->getZ());
        glEnd();

        glBegin(GL_LINES);
            glVertex3d(p1->getX(), p1->getY(), p2->getZ());
            glVertex3d(p2->getX(), p1->getY(), p2->getZ());
        glEnd();


    return (tp2.tv_sec -tp1.tv_sec) * 1000000 + (tp2.tv_usec - tp1.tv_usec);
}

