/***************************************************************************
 *   Copyright (C) 2006 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *   Chemnitz University of Technology                                     *
 *                                                                         *
 *   Implementation of the Dynamic Tubular Grid data structure             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DTGRID_H
#define DTGRID_H

#include <vector>
#include "includes.h"
#include "voxelobject3d.h"
#include "structures.h"

using namespace std;

/**
	@author Holger Gerth <holger.gerth@informatik.tu-chemnitz.de>
*/
class CDtGrid{
public:
    unsigned int            dimension;              // dimension of the DT Grid
    CDtGrid                 *projND;                // adjacent DT Grid with dimension N-1
    CDtGrid                 *parent;                // adjacent DT Grid with dimension N+1
    vector<unsigned int>    acc;                    // the acc array
    vector<int>             nCoord;                 // nCoord array
    vector<double>          value;                  // the value array
    vector<indexPair>       valueIndexPair;         // value array in lower DT Grids

    // for time measurement
    double accessTimeRandomAccess, accessAverageTimeRandomAccess, accessTimeSequentialAccess;

    CDtGrid();
    CDtGrid( unsigned int dimension, CDtGrid *parent = NULL );

    ~CDtGrid();
    void push( vector<int> coordinates, double value );     // push operation
    void push( vector<int> coordinates, indexPair value );
    bool convert_to_dtgrid( CVoxelObject3D* vo );           // create a DT Grid from the given voxel object
    triple randomAccess( int X, int Y, int Z );             // the random access operator
    triple randomAccess( vector<int> coordinates );
    vector<coordinate_value> sequentialAccess();             // the random access operator

    // for testing
    void drawCubes();                               // draw cubes using random access
    void drawCubesUsingSequentialAccess();          // draw cubes usding sequential access
    void printDataStructure();                      // print the content of the data structure
    double getMemorySize();                         // get the used memory

private:

    // create a DT Grid from a Voxel Object
    bool create_dtgrid_from_voxel_object( CVoxelObject3D* vo );

    // for testing
    int minBBox, maxBBox;                   // minimum and maximum extends from the bounding box
    CVertex3D minRange, maxRange;           // minimum and maximum coordinate of the bounding box in the world coordinate system

    void drawACube(CVertex3D *p1, CVertex3D *p2, double colorValue);
    int accessBinSearch( int coord, int kMin, int kMax );

};

#endif
