/***************************************************************************
 *   Lightwave Object Parser                                               *
 *   Copyright (C) 2005 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#if !defined(LIGHTWAVEOBJECT_H_)
#define LIGHTWAVEOBJECT_H_

#include <vector>
#include <fstream>
#include <cstdlib>
#include <iostream>

#include "vertex3d.h"
#include "polygon.h"
#include "surface.h"

/**
 * @short Lightwave Object File Parser
 * @author Holger Gerth <holger.gerth@informatik.tu-chemnitz.de>
 * @version 0.1
*/

using namespace std;

// structure for the PTAG
// typedef struct struct_PTAG {
// 	
// 	unsigned short int a;		// index in the POLS Chunk
// 	unsigned short int b;		// index in the TAGS Chunk
// 
// } ptag;
typedef struct struct_PTAG {
	
	unsigned short int iTag;               // index in the SURF or COLS chunk
	vector<int> vecPolygons;               // List of Indices in the POLS chunk

} ptag;

class CLightwaveObject  
{
    public:

        /////////////////////////// member functions ///////////////////////////////////////
        CLightwaveObject();
        virtual ~CLightwaveObject();

        vector<int> *getSurfacePolygons(char* cSurfaceName);
        CSurface* getSurfaceByName(char* name);
        bool loadObjectFile(const char* filename);

        int getFaceCount() const;
        int getVertexCount() const;
        CVertex3D getBBoxLowerCorner() const;
        CVertex3D getBBoxUpperCorner() const;
        CPolygon getPolygonAtPos( int pos ) const;
        CVertex3D getVertexAtPos( int pos ) const;

    private:

        /////////////////////////// member variables ///////////////////////////////////////

        vector<CVertex3D>       vecVertices;

        CVertex3D               vBB_upperCorner;
        CVertex3D               vBB_lowerCorner;

        vector<CPolygon>        cpFaces;
        vector<int>             vecFaces;

        int                     iVertexCount;
        int                     iFaceCount;

        vector<char*>           vecSurfNames;
        vector<CSurface>        vecSurfaces;

        vector<ptag>            vecPTAG_Color;
        vector<ptag>            vecPTAG_Surface;

        /////////////////////////// member functions ///////////////////////////////////////

        unsigned long littleEndian2BigEndian(unsigned long original);
        unsigned short int littleEndian2BigEndian(unsigned short int original);
        char* littleEndian2BigEndian(char* original);

        void loadPTAG(char* type, unsigned long iChunkSize, ifstream* file);
        void loadSurface(unsigned long ChunkSize, ifstream* file);
        void loadSurfaceNames(ifstream* file, unsigned long iChunkSize);
        void loadVertices(ifstream* file, unsigned long iChunkSize);
        void loadBoundingBox(ifstream* file, unsigned long iChunkSize);
        void loadPolygons(ifstream* file, unsigned long iChunkSize);

        void sortPtag(vector<ptag> *ptag, int start, int end);
        ptag* checkIfSurfaceExists(unsigned short int surface);
};

#endif // !defined(LIGHTWAVEOBJECT_H_)
