
/***************************************************************************
 *   Copyright (C) 2006 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef PERFECTSPATIALHASH_H
#define PERFECTSPATIALHASH_H

#include <vector>
#include "includes.h"
#include "array3d.h"
#include "vertex3d.h"
#include "voxelobject3d.h"

/**
	@author Holger Gerth <holger.gerth@informatik.tu-chemnitz.de>
*/

/*             Important information!
    This version uses 3D arrays instead of textures to save data and is fully implented in software
    not on the graphical hardware using shaders as recommended.
*/

static const int            d = 3;              // spatial dimension

typedef struct pointListStruct {
    CPoint3D                point;          // coordinate of the voxel
    struct pointListStruct  *next;          // next list element
    unsigned int            listSize;       // number of elements in this list
    double                  value;          // numeric value of the voxel
} pointList;

typedef struct structCoordinateIndex{
    CPoint3D            coordinate;         // coordinates
    int                 value;              // the defined value
} coordinateIndex;

class CPerfectSpatialHash{
public:

    CPerfectSpatialHash();
    CPerfectSpatialHash( CVoxelObject3D *vo );
    ~CPerfectSpatialHash();

    CPoint3D computeHash( CPoint3D p, int size );
    float hashedTexture( CPoint3D pf );
    bool createPerfectHashFromVoxelobject( CVoxelObject3D *vo );
    double randomAccess( int x, int y, int z );
    double randomAccess( CPoint3D point );

    // for testing
    double accessTimeRandomAccess;
    double accessAverageTimeRandomAccess;
    double accessTimeSequentialAccess;

    void drawCubes();                               // draw cubes using random access
    void printDataStructure();                      // print the content of the data structure
    double getMemorySize();                         // get the used memory

private:
    CArray3D<CPoint3D>          SOffset;       // hash table
    CArray3D<double>            SHData;        // offset table
    CArray3D<int>               domainBit;     // determines if a grid point is defined or not

    void quicksort( coordinateIndex* array, int min, int max );
    bool checkFreePlaces( pointList* p, CPoint3D point = CPoint3D(0,0,0));

    // for testing
    CPoint3D minBBox, maxBBox;              // minimum and maximum extends from the bounding box
    CVertex3D minRange, maxRange;           // minimum and maximum coordinate of the bounding box in the world coordinate system

    void drawACube(CVertex3D *p1, CVertex3D *p2, double colorValue);
};

#endif
