/***************************************************************************
 *   Qt 4 OpenGL Template                                                  *
 *   Copyright (C) 2006 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "qgltemplate.h"

/*!
    \fn qgltemplate::qgltemplate()
 */
qgltemplate::qgltemplate()
    : QMainWindow( 0, 0 )
{

    // set openglWidget size
    this->setGeometry(50,50,800,600);

    // create OpenGL openglWidget
    openglWidget = new CqOpenGLwidget(this);

    // setup geometry for the OpenGL openglWidget
    openglWidget->setGeometry( this->geometry());

    // create the menu
    createMenu();



    setCentralWidget( openglWidget );
    openglWidget->show();
    statusBar()->showMessage( tr("Ready"), 2000 );


}

/*!
    \fn qgltemplate::~qgltemplate()
 */
qgltemplate::~qgltemplate()
{

    delete newAct;
    delete openAct;
    delete saveAct;
    delete saveAsAct;
    delete closeAct;
    delete quitAct;
    delete aboutAct;
    delete aboutQtAct;
//     delete whatsThisAct;
    delete lightingAct;
    delete coordinatesAct;
    delete wireframeAct;
    delete colorAct;
    delete cullingAct;
    delete transparencyAct;
    delete camera0Act;
    delete camera1Act;
    delete triangleDrawAct;

    delete voxelLoadAct;
    delete voxelDrawAct;

    delete hrleLoadAct;
    delete hrleDrawRandAct;
    delete hrleDrawSeqAct;

    delete hrleRemovePoints;
    delete hrleMemUsage;
    delete hrleGetNeighborAct;
    delete dtLoadAct;
    delete dtDrawRandAct;

    delete phDrawRandAct;
    delete phLoadAct;
    delete phMemUsage;

    delete sparseLoadAct;
    delete sparseDrawRandAct;
    delete sparseDrawSeqAct;
    delete sparseMemUsage;

    delete loadTestObjectAct;

}

// ******************************************************************************
// slots & functions
// ******************************************************************************

/*!
    \fn qgltemplate::newDoc()
 */
void qgltemplate::newDoc()
{
    qgltemplate *ed = new qgltemplate;
    ed->setWindowTitle(tr("Qt OpenGL Template"));
    ed->show();
}

/*!
    \fn qgltemplate::choose()
 */
// file selector for load
void qgltemplate::choose()
{
    QString fn = QFileDialog::getOpenFileName( this, QString::null, QString::null);
    if ( !fn.isEmpty() )
	load( fn );
    else
	statusBar()->showMessage( tr("Loading aborted"), 2000 );
}

/*!
    \fn qgltemplate::load( const QString &filename )

    @param filename
 */
// for loading an smf or lwo file
void qgltemplate::load( const QString &fileName )
{
    QStringList parts = fileName.split('.');
    QByteArray tmp = fileName.toAscii();

    if (parts.contains("smf")) {
        openglWidget->loadSmfObject( tmp.constData() );

        this->triangleDrawAct->setEnabled( true );
        this->voxelLoadAct->setEnabled( true );

        statusBar()->showMessage( tr("Loaded source file %1").arg(fileName), 2000 );
    }
    else {
        statusBar()->showMessage( tr("False file format"), 2000 );
        cerr << "Error: False file format!" << endl;
    }
}

/*!
    \fn qgltemplate::save()
 */
void qgltemplate::save()
{
/// @TODO implement save() slot
//     if ( filename.isEmpty() ) {
// 	saveAs();
// 	return;
//     }
// 
//    QString text = e->text();
//     QFile f( filename );
//     if ( !f.open( IO_WriteOnly ) ) {
// 	statusBar()->showMessage( tr("Could not write to %1").arg(filename),
// 			      2000 );
// 	return;
//     }
// 
//     QTextStream t( &f );
//    t << text;
//     f.close();
// 
//    e->setModified( FALSE );
// 
//     setCaption( filename );
// 
//     statusBar()->showMessage( tr( "File %1 saved" ).arg( filename ), 2000 );
}

/*!
    \fn qgltemplate::saveAs()
 */
void qgltemplate::saveAs()
{
/// @TODO implement saveAs() slot
//     QString fn = QFileDialog::getSaveFileName( QString::null, QString::null,
// 					       this );
//     if ( !fn.isEmpty() ) {
// 	filename = fn;
// 	save();
//     } else {
// 	statusBar()->showMessage( tr("Saving aborted"), 2000 );
//     }
}

/*!
    \fn qgltemplate::closeEvent( QCloseEvent* ce )

    @param ce
 */
void qgltemplate::closeEvent( QCloseEvent* ce )
{
/// @TODO implement closeEvent() function (if needed)
//     if ( !e->isModified() ) {
// 	ce->accept();
// 	return;
//     }
// 
//     switch( QMessageBox::information( this, tr("Qt Application Example"),
// 				      tr("Do you want to save the changes"
// 				      " to the document?"),
// 				      tr("Yes"), tr("No"), tr("Cancel"),
// 				      0, 1 ) ) {
//     case 0:
// 	save();
// 	ce->accept();
// 	break;
//     case 1:
// 	ce->accept();
// 	break;
//     case 2:
//     default: // just for sanity
// 	ce->ignore();
// 	break;
//     }
}

/*!
    \fn qgltemplate::about()
 */
void qgltemplate::about()
{
    QMessageBox::about( this, tr("Qt OpenGL Template"),
			tr("Template for OpenGL with Qt 4."
			"\n(C)2006 Holger Gerth"
			"\nholger.gerth@informatik.tu-chemnitz.de"
			"\n\nVersion for testing the H-RLE level set class."));
}

/*!
    \fn qgltemplate::aboutQt()
 */
void qgltemplate::aboutQt()
{
    QMessageBox::aboutQt( this, tr("Qt OpenGL Template") );
}


/*!
    \fn qgltemplate::createMenu()
 */
void qgltemplate::createMenu()
{
    // create the file menu
    QMenu * file = menuBar()->addMenu(tr("&File"));

        // setup the actions
        newAct = new QAction( tr("&New"), this);
        newAct->setShortcut(tr("Ctrl+N"));
        newAct->setStatusTip(tr("Create a new instance"));
        connect(newAct, SIGNAL(triggered()), this, SLOT(newDoc()));

        openAct = new QAction( tr("&Open..."), this);
        openAct->setShortcut(tr("Ctrl+O"));
        openAct->setStatusTip(tr("Open an existing file"));
        connect(openAct, SIGNAL(triggered()), this, SLOT(choose()));

        saveAct = new QAction( tr("&Save"), this);
        saveAct->setShortcut(tr("Ctrl+S"));
        saveAct->setStatusTip(tr("Save the object"));
        connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));
        saveAct->setDisabled(true);

        saveAsAct = new QAction( tr("Save &As"), this);
        saveAsAct->setStatusTip(tr("Save the object as ..."));
        connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));
        saveAsAct->setDisabled(true);

        closeAct = new QAction( tr("&Close"), this);
        closeAct->setShortcut(tr("Ctrl+W"));
        closeAct->setStatusTip(tr("Close this Widget"));
        connect(closeAct, SIGNAL(triggered()), this, SLOT(close()));

        quitAct = new QAction( tr("&Quit"), this);
        quitAct->setShortcut(tr("Ctrl+Q"));
        quitAct->setStatusTip(tr("Quit the application"));
        connect(quitAct, SIGNAL(triggered()), qApp, SLOT(quit()));
        quitAct->setWhatsThis(tr("Click this option to create a new file."));

        loadTestObjectAct = new QAction( tr("&Load test object"), this);
        loadTestObjectAct->setStatusTip(tr("Load test object"));
        connect(loadTestObjectAct, SIGNAL(triggered()), this, SLOT(loadTestObject()));
        loadTestObjectAct->setWhatsThis(tr("Load a test HLRE object."));

        // insert the actions into the file menu
        file->addAction( newAct );
        file->addAction( openAct );
        file->addAction( loadTestObjectAct );
        file->addAction( saveAct );
        file->addAction( saveAsAct );
        file->addSeparator();
        file->addAction( closeAct );
        file->addAction( quitAct );

        menuBar()->addSeparator();

    // create options menu
    QMenu* options = menuBar()->addMenu( tr( "&Options" ));

        lightingAct = new QAction("&Lighting", this);
        lightingAct->setStatusTip(tr("Activate OpenGL Lighting"));
        connect (lightingAct, SIGNAL (triggered ()), this, SLOT (setLighting ()));
        lightingAct->setCheckable( true );
        lightingAct->setChecked( openglWidget->getLightingStatus() );

        coordinatesAct = new QAction("&Coordinate System", this);
        coordinatesAct->setStatusTip(tr("Activate Coordinate System"));
        connect (coordinatesAct, SIGNAL (triggered ()), this, SLOT (drawCoordinates ()));
        coordinatesAct->setCheckable( true );
        coordinatesAct->setChecked( openglWidget->getCoordinatesStatus() );

        wireframeAct = new QAction("&Wireframe", this);
        wireframeAct->setStatusTip(tr("Activate wire frame mode"));
        connect (wireframeAct, SIGNAL (triggered ()), this, SLOT (setWireFrameOn ()));
        wireframeAct->setCheckable( true );
        wireframeAct->setChecked( openglWidget->getWireFrameStatus() );

        colorAct = new QAction("&ColorMaterial", this);
        colorAct->setStatusTip(tr("Use colormaterial instead of material definitions"));
        connect (colorAct, SIGNAL (triggered ()), this, SLOT (setColorMaterial ()));
        colorAct->setCheckable( true );
        colorAct->setChecked( openglWidget->getColorMaterialStatus() );

        cullingAct = new QAction("&BackFace Culling", this);
        cullingAct->setStatusTip(tr("Activate backface culling"));
        connect (cullingAct, SIGNAL (triggered ()), this, SLOT (setBackFaceCulling ()));
        cullingAct->setCheckable( true );
        cullingAct->setChecked( openglWidget->getCullBackFaceStatus() );

        transparencyAct = new QAction("&Transparency", this);
        transparencyAct->setStatusTip(tr("Activate transparency"));
        QObject::connect (transparencyAct, SIGNAL (triggered ()), this, SLOT (setTransparency ()));
        transparencyAct->setCheckable( true );
        transparencyAct->setChecked( openglWidget->getTransparencyStatus() );

        options->addAction( lightingAct );
        options->addAction( colorAct );
        options->addAction( cullingAct );
        options->addAction( transparencyAct );
        options->addSeparator();
        options->addAction( coordinatesAct );
        options->addAction( wireframeAct );

        menuBar()->addSeparator();

    // create camera menu
    QMenu* cameraMenu = menuBar()->addMenu(tr("&Camera"));

        camera0Act = new QAction("&Free Camera", this);
        camera0Act->setStatusTip(tr("Use the free camera"));
        QObject::connect (camera0Act, SIGNAL (triggered ()), this, SLOT (setCamera1 ()));
        camera0Act->setCheckable( true );

        camera1Act = new QAction("&Centered Camera", this);
        camera1Act->setStatusTip(tr("Use the centered camera"));
        QObject::connect (camera1Act, SIGNAL (triggered ()), this, SLOT (setCamera2 ()));
        camera1Act->setCheckable( true );
        camera1Act->setChecked(true);

        cameraMenu->addAction( camera0Act );
        cameraMenu->addAction( camera1Act );

        menuBar()->addSeparator();

    // create menu for the triangulated object
    QMenu * menuTriangleObject = menuBar()->addMenu(tr("&Object"));

        triangleDrawAct = new QAction( tr("&Draw"), this);
        triangleDrawAct->setStatusTip(tr("Draw the triangulated object"));
        connect(triangleDrawAct, SIGNAL(triggered()), this, SLOT(drawTriangulatedObject()));
        triangleDrawAct->setCheckable( true );
        triangleDrawAct->setDisabled( true );

        menuTriangleObject->addAction( triangleDrawAct );

    // create menu for the voxel object
    QMenu * menuVoxelObject = menuBar()->addMenu(tr("&Voxel Object"));

        voxelLoadAct = new QAction( tr("&Create from triangulated object"), this);
        voxelLoadAct->setStatusTip(tr("Create from triangulated object"));
        connect(voxelLoadAct, SIGNAL(triggered()), this, SLOT(loadVoxelObject()));
        voxelLoadAct->setDisabled( true );

        voxelDrawAct = new QAction( tr("&Draw"), this);
        voxelDrawAct->setStatusTip(tr("Draw the voxel object"));
        connect(voxelDrawAct, SIGNAL(triggered()), this, SLOT(drawVoxelObject()));
        voxelDrawAct->setCheckable( true );
        voxelDrawAct->setDisabled( true );

        voxelMemUsage = new QAction( tr("&Memory and access time"), this);
        voxelMemUsage->setStatusTip(tr("Informations about the data structure"));
        connect(voxelMemUsage, SIGNAL(triggered()), this, SLOT(voMemoryUsage()));
        voxelMemUsage->setDisabled( true );

        menuVoxelObject->addAction( voxelLoadAct );
        menuVoxelObject->addAction( voxelDrawAct );
        menuVoxelObject->addAction( voxelMemUsage );

    // create menu for the H-RLE voxel object
    QMenu * menuHrleObject = menuBar()->addMenu(tr("&H-RLE Object"));

        hrleLoadAct = new QAction( tr("&Create from VoxelObject"), this);
        hrleLoadAct->setStatusTip(tr("Create from VoxelObject"));
        connect(hrleLoadAct, SIGNAL(triggered()), this, SLOT(loadHrleObject()));
        hrleLoadAct->setDisabled( true );

        hrleDrawRandAct = new QAction( tr("&Draw (Random Access)"), this);
        hrleDrawRandAct->setStatusTip(tr("Use Random Access for drawing"));
        connect(hrleDrawRandAct, SIGNAL(triggered()), this, SLOT(drawHrleUsingRandomAccess()));
        hrleDrawRandAct->setCheckable( true );
        hrleDrawRandAct->setDisabled( true );

        hrleDrawSeqAct = new QAction( tr("&Draw (Sequential Access)"), this);
        hrleDrawSeqAct->setStatusTip(tr("Use Sequential Access for drawing"));
        connect(hrleDrawSeqAct, SIGNAL(triggered()), this, SLOT(drawHrleUsingSequentialAccess()));
        hrleDrawSeqAct->setCheckable( true );
        hrleDrawSeqAct->setDisabled( true );

        hrleGetNeighborAct = new QAction( tr("&Neighbor Access"), this);
        hrleGetNeighborAct->setStatusTip(tr("Get the neighbors of a grid point"));
        connect(hrleGetNeighborAct, SIGNAL(triggered()), this, SLOT(getNeighborsSlot()));
        hrleGetNeighborAct->setDisabled( true );

        hrleDilation = new QAction( tr("&Dilation"), this);
        hrleDilation->setStatusTip(tr("Dilate the H-RLE Object"));
        connect(hrleDilation, SIGNAL(triggered()), this, SLOT(dilateHrle()));
        hrleDilation->setDisabled( true );

        hrleRemovePoints = new QAction( tr("&Remove Far Points"), this);
        hrleRemovePoints->setStatusTip(tr("Removes grid points which are to far away"));
        connect(hrleRemovePoints, SIGNAL(triggered()), this, SLOT(removeFarPoints()));
        hrleRemovePoints->setDisabled( true );

        hrleMemUsage = new QAction( tr("&Memory and access time"), this);
        hrleMemUsage->setStatusTip(tr("Informations about the data structure"));
        connect(hrleMemUsage, SIGNAL(triggered()), this, SLOT(hrleMemoryUsage()));
        hrleMemUsage->setDisabled( true );

        menuHrleObject->addAction( hrleLoadAct );
        menuHrleObject->addAction( hrleDrawRandAct );
        menuHrleObject->addAction( hrleDrawSeqAct );
        menuHrleObject->addAction( hrleGetNeighborAct );
        menuHrleObject->addAction( hrleDilation );
        menuHrleObject->addAction( hrleRemovePoints );
        menuHrleObject->addAction( hrleMemUsage );

    // create menu for the DT Grid
    QMenu * menuDtGrid = menuBar()->addMenu(tr("&DT Grid"));

        dtLoadAct = new QAction( tr("&Create from VoxelObject"), this);
        dtLoadAct->setStatusTip(tr("Create from VoxelObject"));
        connect(dtLoadAct, SIGNAL(triggered()), this, SLOT(loadDTGrid()));
        dtLoadAct->setDisabled( true );

        dtDrawRandAct = new QAction( tr("&Draw (Random Access)"), this);
        dtDrawRandAct->setStatusTip(tr("Use Random Access for drawing"));
        connect(dtDrawRandAct, SIGNAL(triggered()), this, SLOT(drawDTGridUsingRandomAccess()));
        dtDrawRandAct->setCheckable( true );
        dtDrawRandAct->setDisabled( true );

        dtDrawSeqAct = new QAction( tr("&Draw (Sequential Access)"), this);
        dtDrawSeqAct->setStatusTip(tr("Use Sequential Access for drawing"));
        connect(dtDrawSeqAct, SIGNAL(triggered()), this, SLOT(drawDTGridUsingSequentialAccess()));
        dtDrawSeqAct->setCheckable( true );
        dtDrawSeqAct->setDisabled( true );

        dtMemUsage = new QAction( tr("&Memory and access time"), this);
        dtMemUsage->setStatusTip(tr("Informations about the data structure"));
        connect(dtMemUsage, SIGNAL(triggered()), this, SLOT(dtMemoryUsage()));
        dtMemUsage->setDisabled( true );

        menuDtGrid->addAction( dtLoadAct );
        menuDtGrid->addAction( dtDrawRandAct );
        menuDtGrid->addAction( dtDrawSeqAct );
        menuDtGrid->addAction( dtMemUsage );

    // create menu for the Perfect Spatial Hash
    QMenu * menuPSHash = menuBar()->addMenu(tr("&PS Hashing"));

        phLoadAct = new QAction( tr("&Create from VoxelObject"), this);
        phLoadAct->setStatusTip(tr("Create from VoxelObject"));
        connect(phLoadAct, SIGNAL(triggered()), this, SLOT(loadPSHash()));
        phLoadAct->setDisabled( true );

        phDrawRandAct = new QAction( tr("&Draw (Random Access)"), this);
        phDrawRandAct->setStatusTip(tr("Use Random Access for drawing"));
        connect(phDrawRandAct, SIGNAL(triggered()), this, SLOT(drawPSHashUsingRandomAccess()));
        phDrawRandAct->setCheckable( true );
        phDrawRandAct->setDisabled( true );

        phMemUsage = new QAction( tr("&Memory and access time"), this);
        phMemUsage->setStatusTip(tr("Informations about the data structure"));
        connect(phMemUsage, SIGNAL(triggered()), this, SLOT(phMemoryUsage()));
        phMemUsage->setDisabled( true );

        menuPSHash->addAction( phLoadAct );
        menuPSHash->addAction( phDrawRandAct );
        menuPSHash->addAction( phMemUsage );

    // create menu for the RLE Sparse Level Set
    QMenu * menuSparse = menuBar()->addMenu(tr("&RLE Sparse Level Set"));

        sparseLoadAct = new QAction( tr("&Create from VoxelObject"), this);
        sparseLoadAct->setStatusTip(tr("Create from VoxelObject"));
        connect(sparseLoadAct, SIGNAL(triggered()), this, SLOT(loadSparseLevelSet()));
        sparseLoadAct->setDisabled( true );

        sparseDrawRandAct = new QAction( tr("&Draw (Random Access)"), this);
        sparseDrawRandAct->setStatusTip(tr("Use Random Access for drawing"));
        connect(sparseDrawRandAct, SIGNAL(triggered()), this, SLOT(drawSparseLevelSetUsingRandomAccess()));
        sparseDrawRandAct->setCheckable( true );
        sparseDrawRandAct->setDisabled( true );

        sparseDrawSeqAct = new QAction( tr("&Draw (Sequential Access)"), this);
        sparseDrawSeqAct->setStatusTip(tr("Use Sequential Access for drawing"));
        connect(sparseDrawSeqAct, SIGNAL(triggered()), this, SLOT(drawSparseLevelSetUsingSequentialAccess()));
        sparseDrawSeqAct->setCheckable( true );
        sparseDrawSeqAct->setDisabled( true );

        sparseMemUsage = new QAction( tr("&Memory and access time"), this);
        sparseMemUsage->setStatusTip(tr("Informations about the data structure"));
        connect(sparseMemUsage, SIGNAL(triggered()), this, SLOT(sparseMemoryUsage()));
        sparseMemUsage->setDisabled( true );

        menuSparse->addAction( sparseLoadAct );
        menuSparse->addAction( sparseDrawRandAct );
        menuSparse->addAction( sparseDrawSeqAct );
        menuSparse->addAction( sparseMemUsage );

    // create the help menu
    QMenu * help = menuBar()->addMenu(tr("&Help"));

        aboutAct = new QAction( tr("&About"), this);
        aboutAct->setShortcut(tr("F1"));
        aboutAct->setStatusTip(tr("About this OpenGL template"));
        connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

        aboutQtAct = new QAction( tr("About &Qt"), this);
        aboutQtAct->setStatusTip(tr("About the Qt application development framework"));
        connect(aboutQtAct, SIGNAL(triggered()), this, SLOT(aboutQt()));

//         whatsThisAct = new QAction( tr("What's &This"), this);
//         whatsThisAct->setShortcut(tr("Shift+F1"));
//         whatsThisAct->setStatusTip(tr("Click on objects you're interested in"));
//         connect(whatsThisAct, SIGNAL(triggered()), this, SLOT(whatsThis()));

        help->addAction( aboutAct );
        help->addAction( aboutQtAct );
//         help->addSeparator();
//         help->addAction( whatsThisAct );

}

/*!
    \fn qgltemplate::setLighting()
 */
// slot to activate Lighting
void qgltemplate::setLighting() {

    openglWidget->setLightingStatus( !openglWidget->getLightingStatus() );

    openglWidget->updateGL();
}

/*!
    \fn qgltemplate::drawCoordinates()
 */
// slot to set coordinate system on
void qgltemplate::drawCoordinates() {

    openglWidget->setCoordinatesStatus( !openglWidget->getCoordinatesStatus() );

    openglWidget->updateGL();
}


/*!
    \fn qgltemplate::setCamera1()
 */
// slot to setup the camera mode 1
void qgltemplate::setCamera1()
{
    // use free camera
    openglWidget->setCameraMode( 1 );

    openglWidget->updateGL();

    camera0Act->setChecked(true);
    camera1Act->setChecked(false);
}

/*!
    \fn qgltemplate::setCamera2()
 */
// slot to setup the camera mode 2
void qgltemplate::setCamera2()
{
    // use centered camera
    openglWidget->setCameraMode( 0 );

    openglWidget->updateGL();

    camera0Act->setChecked(false);
    camera1Act->setChecked(true);
}

/*!
    \fn qgltemplate::setWireFrameOn()
 */
// slot to activate Wireframe mode
void qgltemplate::setWireFrameOn()
{

    openglWidget->setWireFrameStatus( !openglWidget->getWireFrameStatus() );

    openglWidget->updateGL();
}

/*!
    \fn qgltemplate::setColorMAterial()
 */
// slot to activate ColorMaterial mode
void qgltemplate::setColorMaterial()
{
    openglWidget->setColorMaterialStatus( !openglWidget->getColorMaterialStatus() );

    openglWidget->updateGL();
}


/*!
    \fn qgltemplate::setBackFaceCulling()
 */
// slot to enable/disable backface culling
void qgltemplate::setBackFaceCulling()
{
    openglWidget->setCullBackFaceStatus( !openglWidget->getCullBackFaceStatus() );

    openglWidget->updateGL();
}

/*!
    \fn qgltemplate::setTransparency()
 */
// slot to enable/disable transparency
void qgltemplate::setTransparency()
{
    openglWidget->setTransparencyStatus( !openglWidget->getTransparencyStatus() );

    openglWidget->updateGL();

}

/*!
    \fn qgltemplate::drawTriangulatedObject()
 */
void qgltemplate::drawTriangulatedObject() {
    if ( openglWidget->getDrawSmfObject() ) openglWidget->setDrawSmfObject( false );
    else openglWidget->setDrawSmfObject( true );
}

/*!
    \fn qgltemplate::loadVoxelObject()
 */
// create a voxel object from the smf object
void qgltemplate::loadVoxelObject() {

    int numCubes = QInputDialog::getInteger( this, "Please insert number of voxels: ", "Please insert number of voxels", 1000 );

    if ( openglWidget->getVoxelObjectStatus() == false ) {
        this->voxelDrawAct->setEnabled( true );
        this->hrleLoadAct->setEnabled( true );
        this->dtLoadAct->setEnabled( true );
//         this->voxelLoadAct->setDisabled( true );
        this->voxelMemUsage->setEnabled( true );
        this->phLoadAct->setEnabled( true );
        this->sparseLoadAct->setEnabled( true );
    }

    openglWidget->loadVoxelObject( numCubes );

    statusBar()->showMessage( tr("VoxelObject created from boundary object"), 2000 );

}

/*!
    \fn qgltemplate::drawVoxelObject()
 */
// draw the voxel object
void qgltemplate::drawVoxelObject() {

    if ( openglWidget->getDrawVoxelObject() ) openglWidget->setDrawVoxelObject( false );
    else openglWidget->setDrawVoxelObject( true );

}

/*!
    \fn qgltemplate::loadHrleObject()
 */
// load HRLE object from voxel object
void qgltemplate::loadHrleObject() {

    if ( openglWidget->getHrleObjectStatus() == false ) {
        this->hrleDrawRandAct->setEnabled( true );
        this->hrleDrawSeqAct->setEnabled( true );
//         this->hrleLoadAct->setDisabled( true );
        this->hrleDilation->setEnabled( true );
        this->hrleMemUsage->setEnabled( true );
        this->hrleGetNeighborAct->setEnabled( true );
    }
    else {
        this->hrleRemovePoints->setEnabled( false );
    }

    openglWidget->loadHrleObjectFromVoxelObject();

    statusBar()->showMessage( tr("VoxelObject converted into a H-RLE voxel object"), 2000 );
}

/*!
    \fn qgltemplate::drawHrleUsingRandomAccess()
 */
// draw the object via random access
void qgltemplate::drawHrleUsingRandomAccess() {
    if ( openglWidget->getDrawHrleRand() ) openglWidget->setDrawHrleRand( false );
    else openglWidget->setDrawHrleRand( true );

    openglWidget->setDrawHrleSeq( false );
    this->hrleDrawSeqAct->setChecked( false );
}

/*!
    \fn qgltemplate::drawHrleUsingSequentialAccess()
 */
// draw the object via sequential access
void qgltemplate::drawHrleUsingSequentialAccess() {
    if ( openglWidget->getDrawHrleSeq() ) openglWidget->setDrawHrleSeq( false );
    else openglWidget->setDrawHrleSeq( true );

    openglWidget->setDrawHrleRand( false );
    this->hrleDrawRandAct->setChecked( false );
}

/*!
    \fn qgltemplate::dilateHrle()
 */
// dilate th HRLE object
void qgltemplate::dilateHrle() {
    if ( openglWidget->getHrleObjectStatus() == true ) {
        int numPoints = QInputDialog::getInteger( this, "Please insert number of Grid Points: ", "Please insert number of Grid Points", 1 );
        if ( numPoints < 1) numPoints = 1;
        openglWidget->dilateHRLEObject( numPoints );
    }

    statusBar()->showMessage( tr("HRLE object dilated!"), 2000 );

}

/*!
    \fn qgltemplate::removeFarPoints()
 */
// dilate th HRLE object
void qgltemplate::removeFarPoints() {
    if ( openglWidget->getHrleObjectStatus() == true ) {
        double distance = QInputDialog::getDouble( this, "Please insert distance: ", "Please insert distance", 1.0 );
        openglWidget->removeFarPoints( distance );
    }

    statusBar()->showMessage( tr("Far points removed!"), 2000 );

}

/*!
    \fn qgltemplate::loadDTGrid()
 */
void qgltemplate::loadDTGrid() {
    if ( openglWidget->getDtGridObjectStatus() == false ) {
        this->dtDrawRandAct->setEnabled( true );
        this->dtDrawSeqAct->setEnabled( true );
        this->dtMemUsage->setEnabled( true );
    }

    openglWidget->loadDtGridFromVoxelObject();

    statusBar()->showMessage( tr("VoxelObject converted into a DT Grid"), 2000 );
}

/*!
    \fn qgltemplate::drawDTGridUsingRandomAccess()
 */
void qgltemplate::drawDTGridUsingRandomAccess() {
    if ( openglWidget->getDrawDtRand() ) openglWidget->setDrawDtRand( false );
    else openglWidget->setDrawDtRand( true );

    openglWidget->setDrawDtSeq( false );
    this->dtDrawSeqAct->setChecked( false );
}

/*!
    \fn qgltemplate::drawDTGridUsingSequentialAccess()
 */
void qgltemplate::drawDTGridUsingSequentialAccess() {
    if ( openglWidget->getDrawDtSeq() ) openglWidget->setDrawDtSeq( false );
    else openglWidget->setDrawDtSeq( true );

    openglWidget->setDrawDtRand( false );
    this->dtDrawRandAct->setChecked( false );
}

void qgltemplate::getNeighborsSlot() {

    QString message;
    nbsForm *nbs = new nbsForm();

    nbs->draw();

    vector<neighbors> returnedNeighbors = openglWidget->getNeighbors( nbs->x, nbs->y, nbs->z, nbs->axis );

    message.append("The given grid point was (");
    message.append(QString::number(nbs->x));
    message.append(",");
    message.append(QString::number(nbs->z));
    message.append(",");
    message.append(QString::number(nbs->y));
    message.append(")\n");

    message.append("\nSearched for neighbors on ");
    if ( nbs->axis == 3 ) message.append("the encoding axis Z");
    if ( nbs->axis == 2 ) message.append("the encoding axis Y");
    if ( nbs->axis == 1 ) message.append("the encoding axis X");
    else message.append("all encoding axis'.");

    if ( returnedNeighbors.size() > 0 ) {
        for ( int i = 0; i < returnedNeighbors.size(); i++ ) {
            neighbors nb = returnedNeighbors.at(i);

            message.append("\n\nThe grid point has a neighbor on the ");
            if ( nb.dimension == 3 ) message.append(" encoding axis Z");
            if ( nb.dimension == 2 ) message.append(" encoding axis Y");
            if ( nb.dimension == 1 ) message.append(" encoding axis X");
            if ( nb.leftNeighbor ) {
                message.append(" in negative coordinate direction with value ");
                message.append(QString::number(nb.leftValue));
            }
            if ( nb.leftNeighbor && nb.rightNeighbor ) message.append(" and\n");
            if ( nb.rightNeighbor ) {
                message.append(" in positive coordinate direction with value ");
                message.append(QString::number(nb.rightValue));
            }
            message.append(".");
        }
    }
    else {
        message.append("\n\nThis grid point has no neighbors (on the given encoding axis).");
    }

    QMessageBox::information( this, tr("Neighbor Search Result"), message);

}

/*!
    \fn qgltemplate::loadPSHash()
 */
void qgltemplate::loadPSHash() {
    if ( openglWidget->getPerfectHashObjectStatus() == false ) {
        this->phDrawRandAct->setEnabled( true );
        this->phMemUsage->setEnabled( true );
    }

    openglWidget->loadPerfectHashFromVoxelObject();

    statusBar()->showMessage( tr("VoxelObject converted into a Perfect Spatial Hash structure"), 2000 );
}

/*!
    \fn qgltemplate::drawPSHashUsingRandomAccess()
 */
void qgltemplate::drawPSHashUsingRandomAccess() {
    if ( openglWidget->getDrawPHRand() ) openglWidget->setDrawPHRand( false );
    else openglWidget->setDrawPHRand( true );
}

/*!
    \fn qgltemplate::loadSparseLevelSet()
 */
void qgltemplate::loadSparseLevelSet() {
    if ( openglWidget->getSparseLevelSetStatus() == false ) {
        this->sparseDrawRandAct->setEnabled( true );
        this->sparseDrawSeqAct->setEnabled( true );
        this->sparseMemUsage->setEnabled( true );
    }

    openglWidget->loadSparseLevelSetFromVoxelObject();

    statusBar()->showMessage( tr("VoxelObject converted into a RLE Sparse Level set."), 2000 );
}

/*!
    \fn qgltemplate::drawSparseLevelSetUsingRandomAccess()
 */
void qgltemplate::drawSparseLevelSetUsingRandomAccess() {
    if ( openglWidget->getDrawSparseLevelSetRand() ) openglWidget->setDrawSparseLevelSetRand( false );
    else openglWidget->setDrawSparseLevelSetRand( true );

    openglWidget->setDrawSparseLevelSetSeq( false );
    this->sparseDrawSeqAct->setChecked( false );
}

/*!
    \fn qgltemplate::drawSparseLevelSetUsingSequentialAccess()
 */
void qgltemplate::drawSparseLevelSetUsingSequentialAccess() {
    if ( openglWidget->getDrawSparseLevelSetSeq() ) openglWidget->setDrawSparseLevelSetSeq( false );
    else openglWidget->setDrawSparseLevelSetSeq( true );

    openglWidget->setDrawSparseLevelSetRand( false );
    this->sparseDrawRandAct->setChecked( false );
}

void qgltemplate::voMemoryUsage()
{

    double mem = this->openglWidget->getMemoryUsageVoxelobject() / 1024.0;
    double accessTimeRandomAccess = this->openglWidget->getAccessTimeRA_voxelobject();
    double accessAverageTimeRandomAccess = this->openglWidget->getAccessAverageTimeRA_voxelobject();
    double accessTimeSequentialAccess = this->openglWidget->getAccessTimeSA_voxelobject();
    CPoint3D resolution = this->openglWidget->getVoxelobjectResolution();
    int numDefVoxels = this->openglWidget->getNumDefinedVoxels();

    QString message;
    message.append("\nThe full voxel grid uses ");
    if ( mem > 1024.0) {
        mem = mem /1024.0;
        message.append(QString::number(mem));
        message.append(" MBytes of system memory.");
    }
    else {
        message.append(QString::number(mem));
        message.append(" KBytes of system memory.");
    }

    message.append("\n\nLast Random Access was done in ");
    message.append(QString::number(accessTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\nAverage time for Random Access was ");
    message.append(QString::number(accessAverageTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\n\nLast Sequential Access was done in ");
    if ( accessTimeSequentialAccess > 1000000.0 ) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" seconds.");
    }
    else if ( accessTimeSequentialAccess > 1000.0) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" milli seconds.");
    }
    else {
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" micro seconds.");
    }

    message.append("\n\nThis voxel object has a resolution of (");
    message.append(QString::number(resolution.getX()));
    message.append(", ");
    message.append(QString::number(resolution.getY()));
    message.append(", ");
    message.append(QString::number(resolution.getZ()));
    message.append(") voxels with ");
    message.append(QString::number(numDefVoxels));
    message.append(" defined voxels.");

    QMessageBox::information( this, tr("Information about memory and access time"), message);
}

void qgltemplate::hrleMemoryUsage()
{

    double mem = this->openglWidget->getMemoryUsageHRLE() / 1024.0;
    double accessTimeRandomAccess = this->openglWidget->getAccessTimeRA();
    double accessAverageTimeRandomAccess = this->openglWidget->getAccessAverageTimeRA();
    double accessTimeSequentialAccess = this->openglWidget->getAccessTimeSA();

    QString message;
    message.append("\nThe HRLE Object uses ");
    if ( mem > 1024.0) {
        mem = mem /1024.0;
        message.append(QString::number(mem));
        message.append(" MBytes of system memory.");
    }
    else {
        message.append(QString::number(mem));
        message.append(" KBytes of system memory.");
    }

    message.append("\n\nLast Random Access was done in ");
    message.append(QString::number(accessTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\nAverage time for Random Access was ");
    message.append(QString::number(accessAverageTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\n\nLast Sequential Access was done in ");
    if ( accessTimeSequentialAccess > 1000000.0 ) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" seconds.");
    }
    else if ( accessTimeSequentialAccess > 1000.0) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" milli seconds.");
    }
    else {
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" micro seconds.");
    }

    QMessageBox::information( this, tr("Information about memory and access time"), message);
}

void qgltemplate::dtMemoryUsage()
{

    double mem = this->openglWidget->getMemoryUsageDtGrid() / 1024.0;
    double accessTimeRandomAccess = this->openglWidget->getAccessTimeRA_dtgrid();
    double accessAverageTimeRandomAccess = this->openglWidget->getAccessAverageTimeRA_dtgrid();
    double accessTimeSequentialAccess = this->openglWidget->getAccessTimeSA_dtgrid();

    QString message;
    message.append("\nThe DT Grid uses ");
    if ( mem > 1024.0) {
        mem = mem /1024.0;
        message.append(QString::number(mem));
        message.append(" MBytes of system memory.");
    }
    else {
        message.append(QString::number(mem));
        message.append(" KBytes of system memory.");
    }

    message.append("\n\nLast Random Access was done in ");
    message.append(QString::number(accessTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\nAverage time for Random Access was ");
    message.append(QString::number(accessAverageTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\n\nLast Sequential Access was done in ");
    if ( accessTimeSequentialAccess > 1000000.0 ) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" seconds.");
    }
    else if ( accessTimeSequentialAccess > 1000.0) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" milli seconds.");
    }
    else {
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" micro seconds.");
    }

    QMessageBox::information( this, tr("Information about memory and access time"), message);
}

void qgltemplate::phMemoryUsage()
{

    double mem = this->openglWidget->getMemoryUsagePSHash() / 1024.0;
    double accessTimeRandomAccess = this->openglWidget->getAccessTimeRA_pshash();
    double accessAverageTimeRandomAccess = this->openglWidget->getAccessAverageTimeRA_pshash();
    double accessTimeSequentialAccess = this->openglWidget->getAccessTimeSA_pshash();

// cout << "Random Access: " << accessTimeRandomAccess << endl;
// cout << "Average Random Access: " << accessAverageTimeRandomAccess << endl;
// cout << "Sequential Access: " << accessTimeSequentialAccess << endl;

    QString message;
    message.append("\nThe hash tables need ");
    if ( mem > 1024.0) {
        mem = mem /1024.0;
        message.append(QString::number(mem));
        message.append(" MBytes of system memory.");
    }
    else {
        message.append(QString::number(mem));
        message.append(" KBytes of system memory.");
    }

    message.append("\n\nLast Random Access was done in ");
    message.append(QString::number(accessTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\nAverage time for Random Access was ");
    message.append(QString::number(accessAverageTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\n\nLast Sequential Access was done in ");
    if ( accessTimeSequentialAccess > 1000000.0 ) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" seconds.");
    }
    else if ( accessTimeSequentialAccess > 1000.0) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" milli seconds.");
    }
    else {
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" micro seconds.");
    }

    QMessageBox::information( this, tr("Information about memory and access time"), message);
}

void qgltemplate::sparseMemoryUsage()
{

    double mem = this->openglWidget->getMemoryUsageSparse() / 1024.0;
    double accessTimeRandomAccess = this->openglWidget->getAccessTimeRA_sparse();
    double accessAverageTimeRandomAccess = this->openglWidget->getAccessAverageTimeRA_sparse();
    double accessTimeSequentialAccess = this->openglWidget->getAccessTimeSA_sparse();

    QString message;
    message.append("\nThe RLE Sparse Level Set needs ");
    if ( mem > 1024.0) {
        mem = mem /1024.0;
        message.append(QString::number(mem));
        message.append(" MBytes of system memory.");
    }
    else {
        message.append(QString::number(mem));
        message.append(" KBytes of system memory.");
    }

    message.append("\n\nLast Random Access was done in ");
    message.append(QString::number(accessTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\nAverage time for Random Access was ");
    message.append(QString::number(accessAverageTimeRandomAccess));
    message.append(" micro seconds.");

    message.append("\n\nLast Sequential Access was done in ");
    if ( accessTimeSequentialAccess > 1000000.0 ) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" seconds.");
    }
    else if ( accessTimeSequentialAccess > 1000.0) {
        accessTimeSequentialAccess = accessTimeSequentialAccess / 1000.0;
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" milli seconds.");
    }
    else {
        message.append(QString::number(accessTimeSequentialAccess));
        message.append(" micro seconds.");
    }

    QMessageBox::information( this, tr("Information about memory and access time"), message);
}

// *****************************************************************************
// Some events
// *****************************************************************************

void qgltemplate::resizeEvent( QResizeEvent *ev)
{
    QSize newsize = ev->size();

    openglWidget->resize( newsize.width(), newsize.height() );

}

/*!
    \fn qgltemplate::keyPressEvent( QKeyEvent *ev )
 */
void qgltemplate::keyPressEvent( QKeyEvent *ev )
{
    // if key is pressed
    switch ( ev->key() ) {
        default:    break;

    }

    openglWidget->updateGL();
	
}

/*!
    \fn qgltemplate::keyReleaseEvent( QKeyEvent *ev )
 */
void qgltemplate::keyReleaseEvent( QKeyEvent *ev )
{
// if key is released
    switch ( ev->key() ) {
        default:    break;

    }

    openglWidget->updateGL();

}


/////////////////////////////////////////////
//  Testing only
/////////////////////////////////////////////

void qgltemplate::loadTestObject()
{
    openglWidget->createTestingHRLE();
    this->hrleDrawRandAct->setEnabled( true );
    this->hrleDrawSeqAct->setEnabled( true );
    this->hrleLoadAct->setDisabled( true );
    this->hrleDilation->setEnabled( true );
    this->hrleRemovePoints->setEnabled( true );
    this->hrleMemUsage->setEnabled( true );
    this->hrleGetNeighborAct->setEnabled( true );
}



