/***************************************************************************
 *   Copyright (C) 2005 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _CARTESTER_H_
#define _CARTESTER_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qtimer.h>
//Added by qt3to4:
#include <QMouseEvent>

#include "includes.h"
#include "camera.h"
#include "mouse.h"
#include "lightwaveobject.h"
// #include "wavingflag.h"
#include "smfobject.h"
#include "voxelobject3d.h"
#include "hierarchicalrle.h"
#include "dtgrid.h"
#include "perfectspatialhash.h"
#include "rlesparselevelset.h"

/**
 * @short OpenGl Template for Qt
 * @author Holger Gerth <holger.gerth@informatik.tu-chemnitz.de>
 * @version 0.1
 */
 
class CqOpenGLwidget : public QGLWidget
{
    Q_OBJECT

    public:
            //Constructor
            CqOpenGLwidget( QWidget *parent = 0 );

            // Destructor
            ~CqOpenGLwidget();

            bool loadSmfObject( const char* filename );
            bool loadVoxelObject( int numCubes );
            bool loadHrleObjectFromVoxelObject();
            bool dilateHRLEObject( int dilationPoints );
            bool removeFarPoints( double distance );
            void createTestingHRLE();
            bool loadDtGridFromVoxelObject();
            bool loadPerfectHashFromVoxelObject();
            bool loadSparseLevelSetFromVoxelObject();

            // get & set methodes
            void setColorMaterialStatus(bool theValue);
            bool getColorMaterialStatus() const;
            void setCoordinatesStatus(bool theValue);
            bool getCoordinatesStatus() const;
            void setCullBackFaceStatus(bool theValue);
            bool getCullBackFaceStatus() const;
            void setLightingStatus(bool theValue);
            bool getLightingStatus() const;
            void setTransparencyStatus(bool theValue);
            bool getTransparencyStatus() const;
            void setCameraMode(const int& theValue);
            int getCameraMode() const;
            void setWireFrameStatus(bool theValue);
            bool getWireFrameStatus() const;
            void setSmfObjectStatus(bool theValue);
            bool getSmfObjectStatus() const;
            void setVoxelObjectStatus(bool theValue);
            bool getVoxelObjectStatus() const;
            void setHrleObjectStatus(bool theValue);
            bool getHrleObjectStatus() const;
            void setDrawVoxelObject(bool theValue);
            bool getDrawVoxelObject() const;
            void setDrawSmfObject(bool theValue);
            bool getDrawSmfObject() const;
            void setDrawHrleSeq(bool theValue);
            bool getDrawHrleSeq() const;
            void setDrawHrleRand(bool theValue);
            bool getDrawHrleRand() const;
            void setDtGridObjectStatus(bool theValue);
            bool getDtGridObjectStatus() const;
            void setDrawDtRand(bool theValue);
            bool getDrawDtRand() const;
            double getMemoryUsageHRLE();
            vector<neighbors> getNeighbors( int x, int y, int z, int axis );
            bool getDrawDtSeq() const;
            void setDrawDtSeq(bool theValue);
            double getAccessTimeRA();
            double getAccessAverageTimeRA();
            double getAccessTimeSA();
            double getMemoryUsageVoxelobject();
            double getAccessTimeRA_voxelobject();
            double getAccessAverageTimeRA_voxelobject();
            double getAccessTimeSA_voxelobject();
            double getMemoryUsageDtGrid();
            double getAccessTimeRA_dtgrid();
            double getAccessAverageTimeRA_dtgrid();
            double getAccessTimeSA_dtgrid();
            void setPerfectHashObjectStatus(bool theValue);
            bool getPerfectHashObjectStatus() const;
            void setDrawPHRand(bool theValue);
            bool getDrawPHRand() const;
            double getAccessTimeRA_pshash();
            double getAccessAverageTimeRA_pshash();
            double getAccessTimeSA_pshash();
            double getMemoryUsagePSHash();
            int getNumDefinedVoxels();
            CPoint3D getVoxelobjectResolution();
            void setDrawSparseLevelSetRand(bool theValue);
            bool getDrawSparseLevelSetRand() const;
            void setDrawSparseLevelSetSeq(bool theValue);
            bool getDrawSparseLevelSetSeq() const;
            void setSparseLevelSetStatus(bool theValue);
            bool getSparseLevelSetStatus() const;
            double getAccessTimeRA_sparse();
            double getAccessAverageTimeRA_sparse();
            double getAccessTimeSA_sparse();
            double getMemoryUsageSparse();

    protected:
            // protected member variables
            CoglCamera3D        *camera;
            CMouse              myMouse;

            // protected member functions
            virtual void initializeGL();
            virtual void paintGL();

            virtual void mouseMoveEvent( QMouseEvent *ev );
            virtual void mousePressEvent( QMouseEvent *ev );
            virtual void mouseReleaseEvent( QMouseEvent *ev );

    private:
            // private variables
            bool                bCoordinates;
            bool                bLighting;
            int                 iCameraMode;
            bool                bWireFrame;
            bool                bColorMaterial;
            bool                bCullBackFace;;
            bool                bTransparency;

            bool                bSmfObjectStatus;
            bool                bVoxelObjectStatus;
            bool                bHrleObjectStatus;
            bool                bDtGridObjectStatus;
            bool                bPerfectHashObjectStatus;
            bool                bSparseLevelSetStatus;

            bool                bSmfObjectChanged;
            bool                bVoxelObjectChanged;
            bool                bHrleObjectChanged;
            bool                bDtGridObjectChanged;
            bool                bPerfectHashObjectChanged;
            bool                bSparseLevelSetChanged;

            bool                bDrawVoxelObject;
            bool                bDrawSmfObject;
            bool                bDrawHrleRand;
            bool                bDrawHrleSeq;
            bool                bDrawDtRand;
            bool                bDrawDtSeq;
            bool                bDrawPHRand;
            bool                bDrawSparseLevelSetRand;
            bool                bDrawSparseLevelSetSeq;

            QTimer              *idleTimer;

            CsmfObject          *mySmfObject;
            CVoxelObject3D      *myVoxelObject;
            CHierarchicalRLE    *myHRLE;
            CDtGrid             *myDtGrid;
            CPerfectSpatialHash *myPerfectHash;
            CRLESparseLevelSet  *mySparseLevelSet;

             // private members
            void staticListsGL();
            void dynListsGL();
            void drawSurface(char* surface, CLightwaveObject* object);
            void setColor(char* surface, CLightwaveObject* object, float gamma = 1.0, bool on = false);

            // definitions for GL lists
            #define         COORDINATES                 10

            #define         SMFOBJECT                   20
            #define         VOXELOBJECT                 30
            #define         HRLE_RANDOM                 31
            #define         HRLE_SEQUENTIAL             32
            #define         DT_RANDOM                   40
            #define         DT_SEQUENTIAL               41
            #define         PH_RANDOM                   50
            #define         SPARSE_RANDOM               60
            #define         SPARSE_SEQUENTIAL           61

    private slots:
            void idleFuncGL();
            virtual void resizeGL(int w, int h);
};

#endif // _CARTESTER_H_
