/***************************************************************************
 *   Copyright (C) 2006 by Holger Gerth                                      *
 *   holger.gerth@informatik.tu-chemnitz.de                                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RLESPARSELEVELSET_H
#define RLESPARSELEVELSET_H

#include "includes.h"
#include "structures.h"
#include <vector>
#include "vertex3d.h"
#include "voxelobject3d.h"

/**
	@author Holger Gerth <holger.gerth@informatik.tu-chemnitz.de>
*/

typedef struct startCodeStruct {
    RUN_TYPE        type;
    unsigned int    definedIndex;
    int             xCoord;
} startCode;

typedef struct segmentCodeStruct {
    unsigned int    index;
    int             yCoord, zCoord;
} segmentCode;

class CRLESparseLevelSet{
public:
    CRLESparseLevelSet();
    CRLESparseLevelSet( CVoxelObject3D* vo );

    ~CRLESparseLevelSet();

    // random access operator
    run_code randomAccess( int X, int Y, int Z);
    run_code randomAccess( vector<int> qv);

    // for testing
    double accessTimeRandomAccess, accessAverageTimeRandomAccess, accessTimeSequentialAccess;

    void printDataStructure();
    void drawCubes();                               // for testing random access
    void drawCubesUsingSequentialAccess();          // for testing sequential access
    double getMemorySize();

private:
    vector<double>          value;                  // the value array
    vector<startCode>       runStartTable;          // run start table
    vector<segmentCode>     segmentStartTable;      // segment start table
    int                     maxX;                   // maximal x coordinate
//     int                     minY;                   // minimal y coordinate
//     int                     maxY;                   // maximal y coordinate
    int                     numY;                   // count of y coordinates

    bool createRLEfromVoxelObject( CVoxelObject3D* vo );
    int binSearch( int xCoord, unsigned int start, unsigned int end );
    unsigned int binSearchY( int yCoord, unsigned int start, unsigned int end );
    unsigned int binSearchZ( int zCoord, unsigned int start, unsigned int end );

    // for testing
    CPoint3D                minBBox, maxBBox;       // minimum and maximum extends from the bounding box
    CVertex3D               minRange,maxRange;

    void drawACube(CVertex3D *p1, CVertex3D *p2, double colorValue);

};

#endif
