/***************************************************************************
 *   Copyright (C) 2006 by Holger Gerth                                    *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef STRUCTURES_H__
#define STRUCTURES_H__

#include <iostream>
#include <cstdlib>
#include <vector>

using namespace std;

// the three different run types
enum RUN_TYPE {
    RT_NEGATIVE,           // outside the object
    RT_DEFINED,            // the narrow band
    RT_POSITIVE            // inside the object
};

// a run code consists of a type and optional of an index into its value array or in the start array of the next lower hrle
typedef struct struct_run_code {
    RUN_TYPE            type;       // run type
    unsigned int        index;      // index in the value field or in the start indices of the lower hrle level set
} run_code;

// neighbor pair for neighbors search
typedef struct struct_neighbors {
    int                 dimension;      // neighbors of which coordinate? (1 = X, 2 = Y, 3 = Z and so on)
    bool                leftNeighbor;   // true if a neighbor at qi-1 exists
    bool                rightNeighbor;  // true if a neighbor at qi+q exists
    double              leftValue;      // the defined value of the left neighbor (if it exists)
    double              rightValue;     // the defined value of the right neighbor (if it exists)
} neighbors;

typedef struct struct_coordinate_value {
    vector<int>         coordinates;    // coordinates
    unsigned int        index;          // defined data index
    double              value;          // the defined value
} coordinate_value;

typedef struct structIndexPair {

    unsigned int iValue;                    // index in the value array
    unsigned int iCoord;                    // index in the nCoord array

} indexPair;

typedef struct structTriple {

    bool inside;                            // telling if the grid point is inside the tubular grid
    unsigned int iValue;                    // index in the value array (if inside == true)
    double value;                           // value of the grid point (if inside == true)

} triple;

#endif //STRUCTURES_H__
