/***************************************************************************
 *   Copyright (C) 2005 Chemnitz University of Technology                  *
 *   Copyright (C) 2006 Holger Gerth                                       *
 *   holger.gerth@informatik.tu-chemnitz.de                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef VOXELOBJECT3D_H
#define VOXELOBJECT3D_H

#include "includes.h"
#include "color4f.h"
#include "vector3d.h"
#include "point3d.h"
#include "array3d.h"
// #include "moments.h"
#include "cube.h"
#include "smfobject.h"

class CVoxelObject3D {

public:

    int m_ScopeX, m_ScopeY, m_ScopeZ;               // scopes: numbers of cubes in each direction
    CVertex3D m_MinRange, m_MaxRange;               // range
    CArray3D<CCube> m_CubeArray;                    // array of cubes
    CArray3D<CColor4f> m_CubeColorArray;            // array of cube colors
    CArray3D<float> m_DistanceMapArray;             // array of distance maps
    CColor4f m_StandardCubeColor;                   // standard cube color

    // number of defined voxels
    int numDefVoxels;

    // for time measurement
    double accessTimeRandomAccess, accessAverageTimeRandomAccess, accessTimeSequentialAccess;

    // file operations
//     CFileImport FileImport;
//     CFileExport FileExport;

    int m_CenterX, m_CenterY, m_CenterZ;            // center cube (for morphol. op.)
    int m_CentroidX, m_CentroidY, m_CentroidZ;      // centroid
//     CVector3D m_pAxisX, m_pAxisY, m_pAxisZ;         // principal axes

    // drawing options
    bool drawCubes;
    bool drawPrincipalAxes;

    // Constructor / Destructor
    CVoxelObject3D();
    ~CVoxelObject3D();

    // drawing
    void Draw();

    // checking neighborhood
    bool N26(int w1, int h1, int d1, int w2, int h2, int d2);
    bool N18(int w1, int h1, int d1, int w2, int h2, int d2);
    bool N6(int w1, int h1, int d1, int w2, int h2, int d2);

    // recalculate centroid
    void RecalculateCentroid();

    // recalculate Orientation
    void RecalculatePrincipalAxes();

    // label object border voxels
    void LabelBorderVoxels();

    // fill holes in object
    void Fill();

    // copy from another voxel object
    void Copy(CVoxelObject3D *v);

    // transform a triangle mesh to a voxel set
    void TransformTriangleMeshToVoxelSet( CsmfObject *tm, int NbOfCubes );

    // returns the used memory in bytes
    double getMemorySize();

    // sequentiall access
    vector<coordinate_value> sequentialAccess();
    void drawCubesUsingSequentialAccess();

    void countDefinedVoxels();

private:

    void TransformTriangleToCubes(CVertex3D *vA, CVertex3D *vB, CVertex3D *vC);

};

#endif
